import pandas as pd
import pytest
from open_fdd.air_handling_unit.faults.fault_condition_seven import FaultConditionSeven
from open_fdd.air_handling_unit.faults.helper_utils import HelperUtils

"""
To see print statements in pytest run with:
$ py -3.12 -m pytest tests/ahu/test_ahu_fc7.py -rP -s

Supply air temperature too low in full heating.
"""

# Constants
TEST_SUPPLY_DEGF_ERR_THRES = 2.0
TEST_SAT_COL = "supply_air_temp"
TEST_SAT_SETPOINT_COL = "sat_setpoint"
TEST_HEATING_SIG_COL = "heating_sig"
TEST_SUPPLY_VFD_SPEED_COL = "supply_vfd_speed"
ROLLING_WINDOW_SIZE = 5

# Initialize FaultConditionSeven with a dictionary
fault_condition_params = {
    "SUPPLY_DEGF_ERR_THRES": TEST_SUPPLY_DEGF_ERR_THRES,
    "SAT_COL": TEST_SAT_COL,
    "SAT_SETPOINT_COL": TEST_SAT_SETPOINT_COL,
    "HEATING_SIG_COL": TEST_HEATING_SIG_COL,
    "SUPPLY_VFD_SPEED_COL": TEST_SUPPLY_VFD_SPEED_COL,
    "TROUBLESHOOT_MODE": False,  # default value
    "ROLLING_WINDOW_SIZE": ROLLING_WINDOW_SIZE,  # rolling sum window size
}

fc7 = FaultConditionSeven(fault_condition_params)


class TestFaultConditionSeven:

    def fault_df(self) -> pd.DataFrame:
        data = {
            TEST_SAT_COL: [65, 64, 63, 62, 61, 60],
            TEST_SAT_SETPOINT_COL: [70, 70, 70, 70, 70, 70],
            TEST_HEATING_SIG_COL: [0.95, 0.96, 0.97, 0.98, 0.99, 1.0],
            TEST_SUPPLY_VFD_SPEED_COL: [0.5, 0.5, 0.5, 0.5, 0.5, 0.5],
        }
        return pd.DataFrame(data)

    def no_fault_df(self) -> pd.DataFrame:
        data = {
            TEST_SAT_COL: [71, 72, 73, 74, 75, 76],
            TEST_SAT_SETPOINT_COL: [70, 70, 70, 70, 70, 70],
            TEST_HEATING_SIG_COL: [0.95, 0.96, 0.97, 0.98, 0.99, 1.0],
            TEST_SUPPLY_VFD_SPEED_COL: [0.5, 0.5, 0.5, 0.5, 0.5, 0.5],
        }
        return pd.DataFrame(data)

    def test_fault_condition_seven(self):
        results = fc7.apply(self.fault_df())
        actual = results["fc7_flag"].sum()
        expected = 2
        message = f"FC7 fault_df actual is {actual} and expected is {expected}"
        assert actual == expected, message

    def test_no_fault_condition_seven(self):
        results = fc7.apply(self.no_fault_df())
        actual = results["fc7_flag"].sum()
        expected = 0
        message = f"FC7 no_fault_df actual is {actual} and expected is {expected}"
        assert actual == expected, message


if __name__ == "__main__":
    pytest.main()



import pandas as pd
import pandas.api.types as pdtypes
from open_fdd.air_handling_unit.faults.fault_condition import FaultCondition
from open_fdd.air_handling_unit.faults.helper_utils import HelperUtils
import operator
import sys


class FaultConditionSix(FaultCondition):
    """Class provides the definitions for Fault Condition 6.

            This fault related to knowing the design air flow for
            ventilation AHU_MIN_CFM_DESIGN which comes from the
            design mech engineered records where then the fault
            tries to calculate that based on totalized measured
            AHU air flow and outside air fraction calc from
            AHU temp sensors. The fault could flag issues where
            flow stations are either not in calibration, temp
            sensors used in the OA frac calc, or possibly the AHU
            not bringing in design air flow when not operating in
            economizer free cooling modes. Troubleshoot by TAB tech
            verifying flow sensor and temp sensor precisions from
            3rd party sensing tools.

            this fault is confusing if you want to play around
            in py code sandbox try this:
    https://gist.github.com/bbartling/e0fb8427b1e0d148a06e3f09121ed5dc#file-fc6-py
    """

    def __init__(self, dict_):
        self.airflow_err_thres = float
        self.ahu_min_oa_cfm_design = float
        self.outdoor_degf_err_thres = float
        self.return_degf_err_thres = float
        self.oat_rat_delta_min = float
        self.ahu_min_oa_dpr = float
        self.supply_fan_air_volume_col = str
        self.mat_col = str
        self.oat_col = str
        self.rat_col = str
        self.supply_vfd_speed_col = str
        self.economizer_sig_col = str
        self.heating_sig_col = str
        self.cooling_sig_col = str
        self.troubleshoot_mode = bool  # default should be False
        self.rolling_window_size = int

        self.set_attributes(dict_)

    def apply(self, df: pd.DataFrame) -> pd.DataFrame:
        if self.troubleshoot_mode:
            self.troubleshoot_cols(df)

        # check analog outputs [data with units of %] are floats only
        columns_to_check = [
            self.supply_vfd_speed_col,
            self.economizer_sig_col,
            self.heating_sig_col,
            self.cooling_sig_col,
        ]

        for col in columns_to_check:
            self.check_analog_pct(df, [col])

        # create helper columns
        df["rat_minus_oat"] = abs(df[self.rat_col] - df[self.oat_col])
        df["percent_oa_calc"] = (df[self.mat_col] - df[self.rat_col]) / (
            df[self.oat_col] - df[self.rat_col]
        )

        # weed out any negative values
        df["percent_oa_calc"] = df["percent_oa_calc"].apply(lambda x: x if x > 0 else 0)

        df["perc_OAmin"] = (
            self.ahu_min_oa_cfm_design / df[self.supply_fan_air_volume_col]
        )

        df["percent_oa_calc_minus_perc_OAmin"] = abs(
            df["percent_oa_calc"] - df["perc_OAmin"]
        )

        df["combined_check"] = operator.or_(
            # OS 1 htg mode
            (
                (df["rat_minus_oat"] >= self.oat_rat_delta_min)
                & (df["percent_oa_calc_minus_perc_OAmin"] > self.airflow_err_thres)
            )
            # verify ahu is running in OS 1 htg mode in min OA
            & (
                (df[self.heating_sig_col] > 0.0) & (df[self.supply_vfd_speed_col] > 0.0)
            ),  # OR
            # OS 4 mech clg mode
            (
                (df["rat_minus_oat"] >= self.oat_rat_delta_min)
                & (df["percent_oa_calc_minus_perc_OAmin"] > self.airflow_err_thres)
            )
            # verify ahu is running in OS 4 clg mode in min OA
            & (df[self.heating_sig_col] == 0.0)
            & (df[self.cooling_sig_col] > 0.0)
            & (df[self.supply_vfd_speed_col] > 0.0)
            & (df[self.economizer_sig_col] == self.ahu_min_oa_dpr),
        )

        # Rolling sum to count consecutive trues
        rolling_sum = (
            df["combined_check"].rolling(window=self.rolling_window_size).sum()
        )
        # Set flag to 1 if rolling sum equals the window size
        df["fc6_flag"] = (rolling_sum == self.rolling_window_size).astype(int)

        if self.troubleshoot_mode:
            print("Troubleshoot mode enabled - not removing helper columns")
            sys.stdout.flush()
            del df["rat_minus_oat"]
            del df["percent_oa_calc"]
            del df["perc_OAmin"]
            del df["percent_oa_calc_minus_perc_OAmin"]
            del df["combined_check"]

        return df



import pandas as pd
import numpy as np
from open_fdd.air_handling_unit.faults.fault_condition import FaultCondition
from open_fdd.air_handling_unit.faults.helper_utils import HelperUtils
import sys


class FaultConditionNine(FaultCondition):
    """Class provides the definitions for Fault Condition 9.
    Outside air temperature too high in free cooling without
    additional mechanical cooling in economizer mode.
    """

    def __init__(self, dict_):
        self.delta_t_supply_fan = float
        self.outdoor_degf_err_thres = float
        self.supply_degf_err_thres = float
        self.ahu_min_oa_dpr = float
        self.sat_setpoint_col = str
        self.oat_col = str
        self.cooling_sig_col = str
        self.economizer_sig_col = str
        self.troubleshoot_mode = bool  # default should be False
        self.rolling_window_size = int

        self.set_attributes(dict_)

    def apply(self, df: pd.DataFrame) -> pd.DataFrame:
        if self.troubleshoot_mode:
            self.troubleshoot_cols(df)

        # Check analog outputs [data with units of %] are floats only
        columns_to_check = [
            self.economizer_sig_col,
            self.cooling_sig_col,
        ]
        self.check_analog_pct(df, columns_to_check)

        # Create helper columns
        df["oat_minus_oaterror"] = df[self.oat_col] - self.outdoor_degf_err_thres
        df["satsp_delta_saterr"] = (
            df[self.sat_setpoint_col]
            - self.delta_t_supply_fan
            + self.supply_degf_err_thres
        )

        df["combined_check"] = (
            (df["oat_minus_oaterror"] > df["satsp_delta_saterr"])
            # verify AHU is in OS2 only free cooling mode
            & (df[self.economizer_sig_col] > self.ahu_min_oa_dpr)
            & (df[self.cooling_sig_col] < 0.1)
        )

        # Rolling sum to count consecutive trues
        rolling_sum = (
            df["combined_check"].rolling(window=self.rolling_window_size).sum()
        )
        # Set flag to 1 if rolling sum equals the window size
        df["fc9_flag"] = (rolling_sum >= self.rolling_window_size).astype(int)

        if self.troubleshoot_mode:
            print("Troubleshoot mode enabled - not removing helper columns")
            sys.stdout.flush()
            del df["oat_minus_oaterror"]
            del df["satsp_delta_saterr"]
            del df["combined_check"]

        return df



import matplotlib.pyplot as plt
import pandas as pd
import numpy as np
import sys


class FaultCodeFiveReport:
    """Class provides the definitions for Fault Condition 5 Report."""

    def __init__(self, config):
        self.mix_degf_err_thres = config["MIX_DEGF_ERR_THRES"]
        self.supply_degf_err_thres = config["SUPPLY_DEGF_ERR_THRES"]
        self.delta_t_supply_fan = config["DELTA_T_SUPPLY_FAN"]
        self.mat_col = config["MAT_COL"]
        self.sat_col = config["SAT_COL"]
        self.heating_sig_col = config["HEATING_SIG_COL"]
        self.supply_vfd_speed_col = config["SUPPLY_VFD_SPEED_COL"]

    def create_plot(self, df: pd.DataFrame, output_col: str = None):
        if output_col is None:
            output_col = "fc5_flag"

        fig, (ax1, ax2, ax3) = plt.subplots(3, 1, figsize=(25, 8))
        fig.suptitle("Fault Conditions 5 Plot")

        ax1.plot(df.index, df[self.mat_col], color="g", label="Mix Temp")
        ax1.plot(df.index, df[self.sat_col], color="b", label="Supply Temp")
        ax1.legend(loc="best")
        ax1.set_ylabel("°F")

        ax2.plot(df.index, df[self.heating_sig_col], label="Htg Valve", color="r")
        ax2.set_xlabel("Date")
        ax2.set_ylabel("%")
        ax2.legend(loc="best")

        ax3.plot(df.index, df[output_col], label="Fault", color="k")
        ax3.set_xlabel("Date")
        ax3.set_ylabel("Fault Flags")
        ax3.legend(loc="best")

        plt.tight_layout(rect=[0, 0.03, 1, 0.95])
        plt.show()
        plt.close()

    def summarize_fault_times(self, df: pd.DataFrame, output_col: str = None) -> dict:
        if output_col is None:
            output_col = "fc5_flag"

        delta = df.index.to_series().diff()
        summary = {
            "total_days": round(delta.sum() / pd.Timedelta(days=1), 2),
            "total_hours": round(delta.sum() / pd.Timedelta(hours=1)),
            "hours_fc5_mode": round(
                (delta * df[output_col]).sum() / pd.Timedelta(hours=1)
            ),
            "percent_true": round(df[output_col].mean() * 100, 2),
            "percent_false": round((100 - round(df[output_col].mean() * 100, 2)), 2),
            "flag_true_mat": round(
                df[self.mat_col].where(df[output_col] == 1).mean(), 2
            ),
            "flag_true_sat": round(
                df[self.sat_col].where(df[output_col] == 1).mean(), 2
            ),
            "hours_motor_runtime": round(
                (delta * df[self.supply_vfd_speed_col].gt(0.01).astype(int)).sum()
                / pd.Timedelta(hours=1),
                2,
            ),
        }

        return summary

    def create_hist_plot(self, df: pd.DataFrame, output_col: str = None):
        if output_col is None:
            output_col = "fc5_flag"

        # Calculate dataset statistics
        df["hour_of_the_day_fc5"] = df.index.hour.where(df[output_col] == 1)

        # Make hist plots fc5
        fig, ax = plt.subplots(tight_layout=True, figsize=(25, 8))
        ax.hist(df.hour_of_the_day_fc5.dropna())
        ax.set_xlabel("24 Hour Number in Day")
        ax.set_ylabel("Frequency")
        ax.set_title(f"Hour-Of-Day When Fault Flag 5 is TRUE")
        plt.show()
        plt.close()

    def display_report_in_ipython(self, df: pd.DataFrame, output_col: str = "fc5_flag"):
        # Display report content in IPython
        print(
            "Fault Condition 5: Supply air temperature too low; should be higher than mix air"
        )

        # Display plot
        self.create_plot(df, output_col)

        # Display summary statistics
        summary = self.summarize_fault_times(df, output_col)

        for key, value in summary.items():
            formatted_key = key.replace("_", " ")
            print(f"{formatted_key}: {value}")
            sys.stdout.flush()

        fc_max_faults_found = df[output_col].max()
        print("Fault Flag Count: ", fc_max_faults_found)
        sys.stdout.flush()

        if fc_max_faults_found != 0:
            self.create_hist_plot(df, output_col)

            flag_true_mat = round(df[self.mat_col].where(df[output_col] == 1).mean(), 2)
            print("Mix Air Temp Mean When In Fault: ", flag_true_mat)

            flag_true_sat = round(df[self.sat_col].where(df[output_col] == 1).mean(), 2)
            print("Supply Air Temp Mean When In Fault: ", flag_true_sat)

            sys.stdout.flush()

            if summary["percent_true"] > 5.0:
                print(
                    "The percent True metric that represents the amount of time for when the fault flag is True is high indicating the AHU temperature sensors for either the supply or mix temperature are out of calibration. Verify the mixing temperature sensor is not a probe type sensor but a long averaging type sensor that is installed properly inside the AHU mixing chamber to get a good solid true reading of the actual air mixing temperature. Poor duct design may also contribute to not having good air mixing, to troubleshoot install data loggers inside the mixing chamber or take measurements when the AHU is running of different locations in the mixing chamber to spot where better air blending needs to take place."
                )
            else:
                print(
                    "The percent True metric that represents the amount of time for when the fault flag is True is low indicating the AHU temperature sensors are within calibration."
                )

        else:
            print("NO FAULTS FOUND - Skipping time-of-day Histogram plot")
            sys.stdout.flush()



from setuptools import setup, find_packages

def read_long_description(file_path):
    with open(file_path, encoding="utf-8") as f:
        return f.read()

setup(
    name="open_fdd",
    version="0.1.0",
    author="Ben Bartling",
    author_email="ben.bartling@gmail.com",
    description="A package for fault detection and diagnosis in HVAC systems",
    long_description=read_long_description("README.md"),
    long_description_content_type="text/markdown",
    url="https://github.com/bbartling/open-fdd",
    packages=find_packages(include=["open_fdd", "open_fdd.*"]),
    install_requires=[
        "pandas",
        "matplotlib",
        "pytest",
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
)



import pandas as pd
import pandas.api.types as pdtypes
import sys


class SharedUtils:
    @staticmethod
    def float_int_check_err(col):
        err_str = " column failed with a check that the data is a float"
        return str(col) + err_str

    @staticmethod
    def float_max_check_err(col):
        err_str = (
            " column failed with a check that the data is a float between 0.0 and 1.0"
        )
        return str(col) + err_str

    @staticmethod
    def isfloat(num):
        try:
            float(num)
            return True
        except:
            return False

    @staticmethod
    def isLessThanOnePointOne(num):
        try:
            if num <= 1.0:
                return True
        except:
            return False

    @staticmethod
    def convert_to_float(df, col):
        if not pdtypes.is_float_dtype(df[col]):
            try:
                df[col] = df[col].astype(float)
            except ValueError:
                raise TypeError(SharedUtils.float_int_check_err(col))
        return df

    @staticmethod
    def apply_rolling_average_if_needed(df, freq="1min", rolling_window="5min"):
        """Apply rolling average if time difference between consecutive
        timestamps is not greater than the specified frequency.
        """

        print(
            "Warning: If data has a one minute or less sampling frequency a rolling average will be automatically applied"
        )
        sys.stdout.flush()

        time_diff = df.index.to_series().diff().iloc[1:]

        # Calculate median time difference to avoid being affected by outliers
        median_diff = time_diff.median()

        print(
            f"Warning: Median time difference between consecutive timestamps is {median_diff}."
        )
        sys.stdout.flush()

        if median_diff > pd.Timedelta(freq):
            print(f"Warning: Skipping any rolling averaging...")
            sys.stdout.flush()

        else:
            df = df.rolling(rolling_window).mean()
            print(
                f"Warning: A {rolling_window} rolling average has been applied to the data."
            )
            sys.stdout.flush()
        return df



import pandas as pd
import pytest
from open_fdd.air_handling_unit.faults.fault_condition_five import FaultConditionFive
from open_fdd.air_handling_unit.faults.helper_utils import HelperUtils

"""
To see print statements in pytest run with:
$ py -3.12 -m pytest open_fdd/tests/ahu/test_ahu_fc5.py -rP -s

SAT too low; should be higher than MAT in HTG MODE
"""

# Constants
TEST_MIX_DEGF_ERR_THRES = 2.0
TEST_SUPPLY_DEGF_ERR_THRES = 2.0
TEST_DELTA_T_SUPPLY_FAN = 5.0
TEST_MIX_TEMP_COL = "mix_air_temp"
TEST_SUPPLY_TEMP_COL = "supply_air_temp"
TEST_HEATING_COIL_SIG_COL = "heating_sig_col"
TEST_SUPPLY_VFD_SPEED_COL = "supply_vfd_speed"
ROLLING_WINDOW_SIZE = 5

# Initialize FaultConditionFive with a dictionary
fault_condition_params = {
    "MIX_DEGF_ERR_THRES": TEST_MIX_DEGF_ERR_THRES,
    "SUPPLY_DEGF_ERR_THRES": TEST_SUPPLY_DEGF_ERR_THRES,
    "DELTA_T_SUPPLY_FAN": TEST_DELTA_T_SUPPLY_FAN,
    "MAT_COL": TEST_MIX_TEMP_COL,
    "SAT_COL": TEST_SUPPLY_TEMP_COL,
    "HEATING_SIG_COL": TEST_HEATING_COIL_SIG_COL,
    "SUPPLY_VFD_SPEED_COL": TEST_SUPPLY_VFD_SPEED_COL,
    "TROUBLESHOOT_MODE": False,  # default value
    "ROLLING_WINDOW_SIZE": ROLLING_WINDOW_SIZE,  # rolling sum window size
}

fc5 = FaultConditionFive(fault_condition_params)


class TestNoFaultInHtg(object):

    def no_fault_df_in_htg(self) -> pd.DataFrame:
        data = {
            TEST_MIX_TEMP_COL: [40.0, 42.0, 43.0, 41.0, 39.0, 40.0],
            TEST_SUPPLY_TEMP_COL: [80.0, 82.0, 81.0, 83.0, 84.0, 80.0],
            TEST_HEATING_COIL_SIG_COL: [0.55, 0.56, 0.54, 0.55, 0.57, 0.55],
            TEST_SUPPLY_VFD_SPEED_COL: [0.55, 0.56, 0.54, 0.55, 0.57, 0.55],
        }
        return pd.DataFrame(data)

    def test_no_fault_in_htg(self):
        results = fc5.apply(self.no_fault_df_in_htg())
        actual = results["fc5_flag"].sum()
        expected = 0
        message = (
            f"FC5 no_fault_df_in_htg actual is {actual} and expected is {expected}"
        )
        assert actual == expected, message


class TestFaultInHtg(object):

    def fault_df_in_htg(self) -> pd.DataFrame:
        data = {
            TEST_MIX_TEMP_COL: [80.0, 81.0, 79.0, 80.5, 82.0, 80.0],
            TEST_SUPPLY_TEMP_COL: [
                81.0,
                81.0,
                79.0,
                80.5,
                82.0,
                80.0,
            ],  # sim not temp rise
            TEST_HEATING_COIL_SIG_COL: [0.55, 0.56, 0.54, 0.55, 0.57, 0.55],
            TEST_SUPPLY_VFD_SPEED_COL: [0.55, 0.56, 0.54, 0.55, 0.57, 0.55],
        }
        return pd.DataFrame(data)

    def test_fault_in_htg(self):
        results = fc5.apply(self.fault_df_in_htg())
        actual = results["fc5_flag"].sum()
        expected = 2
        message = f"FC5 fault_df_in_htg actual is {actual} and expected is {expected}"
        assert actual == expected, message


class TestNoFaultNoHtg(object):

    def no_fault_df_no_htg(self) -> pd.DataFrame:
        data = {
            TEST_MIX_TEMP_COL: [40.0, 42.0, 43.0, 41.0, 39.0, 40.0],
            TEST_SUPPLY_TEMP_COL: [80.0, 82.0, 81.0, 83.0, 84.0, 80.0],
            TEST_HEATING_COIL_SIG_COL: [0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
            TEST_SUPPLY_VFD_SPEED_COL: [0.55, 0.56, 0.54, 0.55, 0.57, 0.55],
        }
        return pd.DataFrame(data)

    def test_no_fault_no_htg(self):
        results = fc5.apply(self.no_fault_df_no_htg())
        actual = results["fc5_flag"].sum()
        expected = 0
        message = (
            f"FC5 no_fault_df_no_htg actual is {actual} and expected is {expected}"
        )
        assert actual == expected, message


class TestFaultNoHtg(object):

    def fault_df_no_htg(self) -> pd.DataFrame:
        data = {
            TEST_MIX_TEMP_COL: [80.0, 81.0, 79.0, 80.5, 82.0, 80.0],
            TEST_SUPPLY_TEMP_COL: [40.0, 41.0, 42.0, 43.0, 40.5, 40.0],
            TEST_HEATING_COIL_SIG_COL: [0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
            TEST_SUPPLY_VFD_SPEED_COL: [0.55, 0.56, 0.54, 0.55, 0.57, 0.55],
        }
        return pd.DataFrame(data)

    def test_fault_no_htg(self):
        results = fc5.apply(self.fault_df_no_htg())
        actual = results["fc5_flag"].sum()
        expected = 0
        message = f"FC5 fault_df_no_htg actual is {actual} and expected is {expected}"
        assert actual == expected, message


class TestFaultOnInt(object):

    def fault_df_on_output_int(self) -> pd.DataFrame:
        data = {
            TEST_MIX_TEMP_COL: [80.0, 81.0, 79.0, 80.5, 82.0, 80.0],
            TEST_SUPPLY_TEMP_COL: [40.0, 41.0, 42.0, 43.0, 40.5, 40.0],
            TEST_HEATING_COIL_SIG_COL: [0.55, 0.56, 0.54, 0.55, 0.57, 0.55],
            TEST_SUPPLY_VFD_SPEED_COL: [55, 56, 54, 55, 57, 55],
        }
        return pd.DataFrame(data)

    def test_fault_on_int(self):
        with pytest.raises(
            TypeError,
            match=HelperUtils().float_int_check_err(TEST_SUPPLY_VFD_SPEED_COL),
        ):
            fc5.apply(self.fault_df_on_output_int())


class TestFaultOnFloatGreaterThanOne(object):

    def fault_df_on_output_greater_than_one(self) -> pd.DataFrame:
        data = {
            TEST_MIX_TEMP_COL: [80.0, 81.0, 79.0, 80.5, 82.0, 80.0],
            TEST_SUPPLY_TEMP_COL: [40.0, 41.0, 42.0, 43.0, 40.5, 40.0],
            TEST_HEATING_COIL_SIG_COL: [0.55, 0.56, 0.54, 0.55, 0.57, 0.55],
            TEST_SUPPLY_VFD_SPEED_COL: [1.1, 1.2, 1.1, 1.3, 1.1, 1.2],
        }
        return pd.DataFrame(data)

    def test_fault_on_float_greater_than_one(self):
        with pytest.raises(
            TypeError,
            match=HelperUtils().float_max_check_err(TEST_SUPPLY_VFD_SPEED_COL),
        ):
            fc5.apply(self.fault_df_on_output_greater_than_one())


class TestFaultOnMixedTypes(object):

    def fault_df_on_mixed_types(self) -> pd.DataFrame:
        data = {
            TEST_MIX_TEMP_COL: [80.0, 81.0, 79.0, 80.5, 82.0, 80.0],
            TEST_SUPPLY_TEMP_COL: [40.0, 41.0, 42.0, 43.0, 40.5, 40.0],
            TEST_HEATING_COIL_SIG_COL: [0.55, 0.56, 0.54, 0.55, 0.57, 0.55],
            TEST_SUPPLY_VFD_SPEED_COL: [1.1, 0.55, 1.2, 1.3, 0.55, 1.1],
        }
        return pd.DataFrame(data)

    def test_fault_on_mixed_types(self):
        with pytest.raises(
            TypeError,
            match=HelperUtils().float_max_check_err(TEST_SUPPLY_VFD_SPEED_COL),
        ):
            fc5.apply(self.fault_df_on_mixed_types())



import pandas as pd
import pytest
from open_fdd.air_handling_unit.faults.fault_condition_eleven import (
    FaultConditionEleven,
)
from open_fdd.air_handling_unit.faults.helper_utils import HelperUtils

"""
To see print statements in pytest run with:
$ py -3.12 -m pytest tests/ahu/test_ahu_fc11.py -rP -s

OAT Temp too low for 100% cooling in OS3
"""

# Constants
TEST_DELTA_SUPPLY_FAN = 2.0
TEST_OAT_DEGF_ERR_THRES = 5.0
TEST_SUPPLY_DEGF_ERR_THRES = 2.0
TEST_SAT_SP_COL = "supply_air_sp_temp"
TEST_OAT_COL = "out_air_temp"
TEST_COOLING_COIL_SIG_COL = "cooling_sig_col"
TEST_MIX_AIR_DAMPER_COL = "economizer_sig_col"
ROLLING_WINDOW_SIZE = 5

# Initialize FaultConditionEleven with a dictionary
fault_condition_params = {
    "DELTA_T_SUPPLY_FAN": TEST_DELTA_SUPPLY_FAN,
    "OUTDOOR_DEGF_ERR_THRES": TEST_OAT_DEGF_ERR_THRES,
    "SUPPLY_DEGF_ERR_THRES": TEST_SUPPLY_DEGF_ERR_THRES,
    "SAT_SETPOINT_COL": TEST_SAT_SP_COL,
    "OAT_COL": TEST_OAT_COL,
    "COOLING_SIG_COL": TEST_COOLING_COIL_SIG_COL,
    "ECONOMIZER_SIG_COL": TEST_MIX_AIR_DAMPER_COL,
    "TROUBLESHOOT_MODE": False,
    "ROLLING_WINDOW_SIZE": ROLLING_WINDOW_SIZE,
}

fc11 = FaultConditionEleven(fault_condition_params)


class TestFaultConditionEleven:

    def no_fault_df_no_econ(self) -> pd.DataFrame:
        data = {
            TEST_SAT_SP_COL: [55, 55, 55, 55, 55, 55],
            TEST_OAT_COL: [56, 56, 56, 56, 56, 56],
            TEST_COOLING_COIL_SIG_COL: [0.11, 0.11, 0.11, 0.11, 0.11, 0.11],
            TEST_MIX_AIR_DAMPER_COL: [0.99, 0.99, 0.99, 0.99, 0.99, 0.99],
        }
        return pd.DataFrame(data)

    def fault_df_in_econ(self) -> pd.DataFrame:
        data = {
            TEST_SAT_SP_COL: [55, 55, 55, 55, 55, 55],
            TEST_OAT_COL: [44, 44, 44, 44, 44, 44],
            TEST_COOLING_COIL_SIG_COL: [0.11, 0.11, 0.11, 0.11, 0.11, 0.11],
            TEST_MIX_AIR_DAMPER_COL: [0.99, 0.99, 0.99, 0.99, 0.99, 0.99],
        }
        return pd.DataFrame(data)

    def test_no_fault_no_econ(self):
        results = fc11.apply(self.no_fault_df_no_econ())
        actual = results["fc11_flag"].sum()
        expected = 0
        message = (
            f"FC11 no_fault_df_no_econ actual is {actual} and expected is {expected}"
        )
        assert actual == expected, message

    def test_fault_in_econ(self):
        results = fc11.apply(self.fault_df_in_econ())
        actual = results["fc11_flag"].sum()
        expected = 2
        message = f"FC11 fault_df_in_econ actual is {actual} and expected is {expected}"
        assert actual == expected, message


class TestFaultOnInt:

    def fault_df_on_output_int(self) -> pd.DataFrame:
        data = {
            TEST_SAT_SP_COL: [55, 55, 55, 55, 55, 55],
            TEST_OAT_COL: [44, 44, 44, 44, 44, 44],
            TEST_COOLING_COIL_SIG_COL: [11, 11, 11, 11, 11, 11],  # Incorrect type
            TEST_MIX_AIR_DAMPER_COL: [0.99, 0.99, 0.99, 0.99, 0.99, 0.99],
        }
        return pd.DataFrame(data)

    def test_fault_on_int(self):
        with pytest.raises(
            TypeError,
            match=HelperUtils().float_int_check_err(TEST_COOLING_COIL_SIG_COL),
        ):
            fc11.apply(self.fault_df_on_output_int())


class TestFaultOnFloatGreaterThanOne:

    def fault_df_on_output_greater_than_one(self) -> pd.DataFrame:
        data = {
            TEST_SAT_SP_COL: [55, 55, 55, 55, 55, 55],
            TEST_OAT_COL: [44, 44, 44, 44, 44, 44],
            TEST_COOLING_COIL_SIG_COL: [1.1, 1.2, 1.1, 1.3, 1.1, 1.2],  # Values > 1.0
            TEST_MIX_AIR_DAMPER_COL: [0.99, 0.99, 0.99, 0.99, 0.99, 0.99],
        }
        return pd.DataFrame(data)

    def test_fault_on_float_greater_than_one(self):
        with pytest.raises(
            TypeError,
            match=HelperUtils().float_max_check_err(TEST_COOLING_COIL_SIG_COL),
        ):
            fc11.apply(self.fault_df_on_output_greater_than_one())


class TestFaultOnMixedTypes:

    def fault_df_on_mixed_types(self) -> pd.DataFrame:
        data = {
            TEST_SAT_SP_COL: [55, 55, 55, 55, 55, 55],
            TEST_OAT_COL: [44, 44, 44, 44, 44, 44],
            TEST_COOLING_COIL_SIG_COL: [1.1, 0.55, 1.2, 1.3, 0.55, 1.1],  # Mixed types
            TEST_MIX_AIR_DAMPER_COL: [0.99, 0.99, 0.99, 0.99, 0.99, 0.99],
        }
        return pd.DataFrame(data)

    def test_fault_on_mixed_types(self):
        with pytest.raises(
            TypeError,
            match=HelperUtils().float_max_check_err(TEST_COOLING_COIL_SIG_COL),
        ):
            fc11.apply(self.fault_df_on_mixed_types())


if __name__ == "__main__":
    pytest.main()
